@@ -152,7 +152,11 @@ static void dr_msg_cb (rd_kafka_t *rk, const rd_kafka_message_t *rkm,
 		goto done;
 	}
 
-	msgobj = Message_new0(rkm);
+        /* Skip callback if delivery.report.only.error=true */
+        if (self->u.Producer.dr_only_error && !rkm->err)
+                goto done;
+
+	msgobj = Message_new0(self, rkm);
 	
 	args = Py_BuildValue("(OO)",
 			     Message_error((Message *)msgobj, NULL),
@@ -324,9 +328,11 @@ static PyObject *Producer_produce (Handle *self, PyObject *args,
 #if !HAVE_PRODUCEV
         if (timestamp) {
                 PyErr_Format(PyExc_NotImplementedError,
-                             "Producer timestamps require librdkafka "
-                             "version >=v0.9.4 (currently on %s)",
-                             rd_kafka_version_str());
+                             "Producer timestamps require "
+                             "confluent-kafka-python built for librdkafka "
+                             "version >=v0.9.4 (librdkafka runtime 0x%x, "
+                             "buildtime 0x%x)",
+                             rd_kafka_version(), RD_KAFKA_VERSION);
                 return NULL;
         }
 #endif
@@ -527,6 +533,8 @@ static int Producer_init (PyObject *selfobj, PyObject *args, PyObject *kwargs) {
                 return -1;
         }
 
+        self->type = RD_KAFKA_PRODUCER;
+
         if (!(conf = common_conf_setup(RD_KAFKA_PRODUCER, self,
                                        args, kwargs)))
                 return -1;
