@@ -258,7 +258,10 @@ PyObject *KafkaError_new0 (rd_kafka_resp_err_t err, const char *fmt, ...) {
  PyObject *KafkaError_new_or_None (rd_kafka_resp_err_t err, const char *str) {
 	if (!err)
 		Py_RETURN_NONE;
-	return KafkaError_new0(err, "%s", str);
+        if (str)
+                return KafkaError_new0(err, "%s", str);
+        else
+                return KafkaError_new0(err, NULL);
 }
 
 
@@ -529,17 +532,19 @@ PyTypeObject MessageType = {
 /**
  * @brief Internal factory to create Message object from message_t
  */
-PyObject *Message_new0 (const rd_kafka_message_t *rkm) {
+PyObject *Message_new0 (const Handle *handle, const rd_kafka_message_t *rkm) {
 	Message *self;
 
 	self = (Message *)MessageType.tp_alloc(&MessageType, 0);
 	if (!self)
 		return NULL;
 
-	self->error = KafkaError_new_or_None(rkm->err,
-					     rkm->err ?
-					     rd_kafka_message_errstr(rkm) :
-					     NULL);
+        /* Only use message error string on Consumer, for Producers
+         * it will contain the original message payload. */
+        self->error = KafkaError_new_or_None(
+                rkm->err,
+                (rkm->err && handle->type != RD_KAFKA_PRODUCER) ?
+                rd_kafka_message_errstr(rkm) : NULL);
 
 	if (rkm->rkt)
 		self->topic = cfl_PyUnistr(
@@ -1119,7 +1124,26 @@ static int producer_conf_set_special (Handle *self, rd_kafka_conf_t *conf,
 		}
 
 		return 1;
-	}
+
+        } else if (!strcmp(name, "delivery.report.only.error")) {
+                /* Since we allocate msgstate for each produced message
+                 * with a callback we can't use delivery.report.only.error
+                 * as-is, as we wouldn't be able to ever free those msgstates.
+                 * Instead we shortcut this setting in the Python client,
+                 * providing the same functionality from dr_msg_cb trampoline.
+                 */
+
+                if (!PyBool_Check(valobj)) {
+                        cfl_PyErr_Format(
+                                RD_KAFKA_RESP_ERR__INVALID_ARG,
+                                "%s requires bool", name);
+                        return -1;
+                }
+
+                self->u.Producer.dr_only_error = valobj == Py_True;
+
+                return 1;
+        }
 
 	return 0; /* Not handled */
 }
@@ -1278,16 +1302,21 @@ rd_kafka_conf_t *common_conf_setup (rd_kafka_type_t ktype,
 		/*
 		 * Pass configuration property through to librdkafka.
 		 */
-		if (!(vs = cfl_PyObject_Unistr(vo))) {
-			PyErr_SetString(PyExc_TypeError,
-					"expected configuration property "
-					"value as type unicode string");
-			rd_kafka_topic_conf_destroy(tconf);
-			rd_kafka_conf_destroy(conf);
-			Py_DECREF(ks);
-			return NULL;
-		}
-		v = cfl_PyUnistr_AsUTF8(vs);
+                if (vo == Py_None) {
+                        v = NULL;
+                } else {
+                        if (!(vs = cfl_PyObject_Unistr(vo))) {
+                                PyErr_SetString(PyExc_TypeError,
+                                                "expected configuration "
+                                                "property value as type "
+                                                "unicode string");
+                                rd_kafka_topic_conf_destroy(tconf);
+                                rd_kafka_conf_destroy(conf);
+                                Py_DECREF(ks);
+                                return NULL;
+                        }
+                        v = cfl_PyUnistr_AsUTF8(vs);
+                }
 
 		if (rd_kafka_conf_set(conf, k, v, errstr, sizeof(errstr)) !=
 		    RD_KAFKA_CONF_OK) {
@@ -1295,12 +1324,12 @@ rd_kafka_conf_t *common_conf_setup (rd_kafka_type_t ktype,
 					  "%s", errstr);
 			rd_kafka_topic_conf_destroy(tconf);
 			rd_kafka_conf_destroy(conf);
-			Py_DECREF(vs);
+                        Py_XDECREF(vs);
 			Py_DECREF(ks);
 			return NULL;
 		}
 
-		Py_DECREF(vs);
+                Py_XDECREF(vs);
 		Py_DECREF(ks);
 	}
 
@@ -1398,7 +1427,7 @@ static PyObject *libversion (PyObject *self, PyObject *args) {
 }
 
 static PyObject *version (PyObject *self, PyObject *args) {
-	return Py_BuildValue("si", "0.9.2", 0x00090100);
+	return Py_BuildValue("si", "0.11.0", 0x000b0000);
 }
 
 static PyMethodDef cimpl_methods[] = {
