@@ -105,6 +105,7 @@ PyObject *KafkaError_new_or_None (rd_kafka_resp_err_t err, const char *str);
 #define cfl_PyErr_Format(err,...) do {					\
 		PyObject *_eo = KafkaError_new0(err, __VA_ARGS__);	\
 		PyErr_SetObject(KafkaException, _eo);			\
+		Py_DECREF(_eo); \
 	} while (0)
 
 
@@ -125,6 +126,7 @@ typedef struct {
 	PyObject *stats_cb;
         int initiated;
         int tlskey;  /* Thread-Local-Storage key */
+        rd_kafka_type_t type; /* Producer or consumer */
 
 	union {
 		/**
@@ -137,6 +139,8 @@ typedef struct {
 				const rd_kafka_topic_t *,
 				const void *, size_t, int32_t,
 				void *, void *);  /**< Fallback C partitioner*/
+
+                        int dr_only_error; /**< delivery.report.only.error */
 		} Producer;
 
 		/**
@@ -255,7 +259,7 @@ typedef struct {
 
 extern PyTypeObject MessageType;
 
-PyObject *Message_new0 (const rd_kafka_message_t *rkm);
+PyObject *Message_new0 (const Handle *handle, const rd_kafka_message_t *rkm);
 PyObject *Message_error (Message *self, PyObject *ignore);
 
 
